/*
  This is an example to demonstrate the choice of appropriate threads and blocks.
*/
#include <stdio.h>

__global__ void example_kernel(float *data)
{
  int i;
  float temp = 0.0f;

  for( i = 0; i < 10000; i++ )
    temp = temp + threadIdx.x + blockIdx.x;

  data[blockIdx.x * blockDim.x + threadIdx.x] = temp;
  
}

int main(int argc, char *argv[])
{
  int numThreads = 256;
  int numBlocks = 8192;

  if( argc > 1 ) numThreads = atof( argv[1] );

  float *cudaData;

  cudaMalloc( (void **)&cudaData, numThreads * numBlocks * sizeof(float) );
  cudaMemset( cudaData, 0, numThreads * numBlocks * sizeof(float) );

  cudaEvent_t start, stop;
  float time;
  cudaEventCreate(&start);
  cudaEventCreate(&stop);

  cudaEventRecord(start, 0);

  dim3 threads(numThreads, 1);
  dim3 blocks(numBlocks, 1);

  example_kernel<<< blocks, threads >>>(cudaData);

  cudaEventRecord(stop, 0);
  cudaEventSynchronize(stop);

  cudaEventElapsedTime( &time, start, stop );

  printf( "Execution time: %f (ms)\n", time );

  cudaEventDestroy(stop);

  cudaFree(cudaData);

  return 0;
}
