/*
  ex34_1.cu 
  This is an example to demonstrate the effect of strided access.
  This software contains source code provided by NVIDIA Corporation.
*/
#include <stdio.h>

#define STRIDELIMIT 512
__global__ void strideCopy(float *odata, float *idata, int stride)
{
  int xid = (blockIdx.x * blockDim.x + threadIdx.x) * stride;

  odata[xid] = idata[xid];
}

int main(int argc, char *argv[])
{
  int i;
  int arrSize = 1024 * 1024 * STRIDELIMIT;
  int stride = 1;

  float *input;

  float *cudaInput;
  float *cudaOutput;

  if( argc > 1 ) stride = atoi(argv[1]);

  input = (float *)malloc( arrSize * sizeof(float) );

  cudaMalloc( (void **)&cudaInput, arrSize * sizeof(float) );
  cudaMalloc( (void **)&cudaOutput, arrSize * sizeof(float) );

  /* Initialize array */
  for( i = 0; i < arrSize; i++ )
    input[i] = (float) (i * 0.1);

  cudaMemcpy( cudaInput, input, arrSize * sizeof(float), 
      cudaMemcpyHostToDevice );

  dim3 threads( 512, 1 );
  dim3 blocks( (arrSize / STRIDELIMIT) / threads.x, 1 );
  
  strideCopy<<< blocks, threads >>>(cudaOutput, cudaInput, stride);

  cudaFree(cudaInput);
  cudaFree(cudaOutput);

  return 0;
}

