/*
   ex42.cu
   This example demonstrate how the bank conflict occurs when reading from shared memory.
*/
#include <stdio.h>

/* To avoid the need to load some extra libraries and use extra functions, we will define our own struct */
typedef struct
{
  float re;
  float im;
} COMP;

/* Kernel for accessing the complex number */
__global__ void simple_kernel(COMP *input, COMP *output, int niterations)
{
  int j;
  int i = blockIdx.x * blockDim.x + threadIdx.x;

  for( j = 0; j < niterations; j++ )
  {
    output[i].re += input[i].re;
    output[i].im += output[i].im;
  }
}

int main(int argc, char *argv[])
{
  int i;

  int niterations = 10;
  
  int nthreads = 256;
  int nblocks = 2048;

  int arrSize = nthreads * nblocks;

  COMP *input;

  COMP *cudaInput;
  COMP *cudaOutput;

  input = (COMP *)malloc( arrSize * sizeof(COMP) );

  cudaMalloc( (void **)&cudaInput, arrSize * sizeof(COMP) );
  cudaMalloc( (void **)&cudaOutput, arrSize * sizeof(COMP) );

  /* Initialize input array */
  for( i = 0; i < arrSize; i++ )
  {
    input[i].re = (float) (i + 1);
    input[i].re = (float) (i - 10);
  }

  cudaMemcpy( cudaInput, input, arrSize * sizeof(COMP), 
      cudaMemcpyHostToDevice );

  simple_kernel<<< nblocks, nthreads >>>(cudaInput, cudaOutput, niterations);
  free(input);
  cudaFree(cudaInput);
  cudaFree(cudaOutput);

  return 0;
}
